@@ -34,5 +34,5 @@ void test_submodule_repository_init__basic(void)
 
 	git_submodule_free(sm);
 	git_repository_free(repo);
-	git_buf_free(&dot_git_content);
+	git_buf_dispose(&dot_git_content);
 }
